<?php

namespace Drupal\oauth2_rs\AuthorizationValidators;

use BadMethodCallException;
use InvalidArgumentException;
use Lcobucci\JWT\Parser;
use Lcobucci\JWT\Signer\Rsa\Sha256;
use Lcobucci\JWT\ValidationData;
use League\OAuth2\Server\AuthorizationValidators\AuthorizationValidatorInterface;
use League\OAuth2\Server\CryptKey;
use League\OAuth2\Server\CryptTrait;
use League\OAuth2\Server\Exception\OAuthServerException;
use Psr\Http\Message\ServerRequestInterface;
use RuntimeException;

/**
 * Validation of given token, whether it's valid.
 */
class BearerTokenValidator implements AuthorizationValidatorInterface {
  use CryptTrait;

  /**
   * Public key.
   *
   * @var \League\OAuth2\Server\CryptKey
   */
  protected $publicKey;

  /**
   * Set the public key.
   *
   * @param \League\OAuth2\Server\CryptKey $key
   *   Public key.
   */
  public function setPublicKey(CryptKey $key) {
    $this->publicKey = $key;
  }

  /**
   * {@inheritdoc}
   */
  public function validateAuthorization(ServerRequestInterface $request) {
    if ($request->hasHeader('authorization') === FALSE) {
      throw OAuthServerException::accessDenied('Missing "Authorization" header');
    }

    $header = $request->getHeader('authorization');
    $jwt = trim(preg_replace('/^(?:\s+)?Bearer\s/', '', $header[0]));

    try {
      // Attempt to parse and validate the JWT.
      $token = (new Parser())->parse($jwt);
      try {
        if ($token->verify(new Sha256(), $this->publicKey->getKeyPath()) === FALSE) {
          throw OAuthServerException::accessDenied('Access token could not be verified');
        }
      }
      catch (BadMethodCallException $exception) {
        throw OAuthServerException::accessDenied('Access token is not signed', NULL, $exception);
      }

      // Ensure access token hasn't expired.
      $data = new ValidationData();
      $data->setCurrentTime(time());

      if ($token->validate($data) === FALSE) {
        throw OAuthServerException::accessDenied('Access token is invalid');
      }
      // Return the request with additional attributes.
      return $request
        ->withAttribute('token', $token)
        ->withAttribute('oauth_access_token_id', $token->getClaim('jti'))
        ->withAttribute('oauth_client_id', $token->getClaim('aud'))
        ->withAttribute('oauth_user_id', $token->getClaim('sub'))
        // TODO get claim names from config?.
        ->withAttribute('oauth_scope', $token->getClaim('scope'))
        ->withAttribute('oauth_username', $token->getClaim('username'))
        ->withAttribute('oauth_user_email', $token->getClaim('email'));
    }
    catch (InvalidArgumentException $exception) {
      // JWT couldn't be parsed so return the request as is.
      throw OAuthServerException::accessDenied($exception->getMessage(), NULL, $exception);
    }
    catch (RuntimeException $exception) {
      // JWR couldn't be parsed so return the request as is.
      throw OAuthServerException::accessDenied('Error while decoding to JSON', NULL, $exception);
    }
  }

}
